/**
 * <p>
 * ASTElement
 * </p>
 * 
 * <p>
 * Used by the FileParser, represents a node in the parse tree. Used to wrap
 * values moved from the parser to the rest of the program.
 * </p>
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class ASTElement extends SimpleNode implements Element {
	IsotopeList isotopes;

	String symbol;

	String name;

	int atomic_number;

	/**
	 * Constructor
	 */
	public ASTElement(int id) {
		super(id);
	}

	/**
	 * Constructor
	 */
	public ASTElement(FileParser p, int id) {
		super(p, id);
	}

	/**
	 * @return the symbol read e.g. <code>C</code>
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * @return the name e.g. <code>Carbon</code>
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the atomic number
	 */
	public int getAtomicNumber() {
		return atomic_number;
	}

	/**
	 * @return a referance to this elements IsotopeList
	 */
	public IsotopeList getIsotopes() {
		return isotopes;
	}

	/**
	 * Called by the parser to initilise the wrapped values
	 * 
	 * @param s
	 *            String representing the symbol
	 * @param n
	 *            String representing the name
	 * @param a_n
	 *            int representing the atomic number
	 */
	public void init(String s, String n, int a_n) {
		symbol = s;
		char[] tmp = n.toLowerCase().toCharArray();
		tmp[0] = Character.toUpperCase(tmp[0]);
		name = new String(tmp);
		atomic_number = a_n;
		isotopes = new IsotopeList(3);
	}

	/**
	 * Overriden method; used when dumping the tree
	 */
	public String toString() {
		return "Element: " + name + " (" + symbol + "), Atomic Number: "
				+ atomic_number;
	}
}
