/**
 * <p>
 * ASTIsotope
 * </p>
 * 
 * <p>
 * Used by the FileParser, represents a node in the parse tree. Used to wrap
 * values moved from the parser to the rest of the program.
 * </p>
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class ASTIsotope extends SimpleNode implements Isotope {
	double percent, weight;

	boolean modified;

	public ASTIsotope(int id) {
		super(id);
	}

	public ASTIsotope(FileParser p, int id) {
		super(p, id);
	}

	/**
	 * initilises the values for this isotope
	 * 
	 * @param p
	 *            percentage
	 * @param w
	 *            atomic weight of isotope
	 */
	public void init(double p, double w) {
		percent = p;
		weight = w;
		modified = false;
	}

	/**
	 * @return the atomic mass
	 */
	public double getAtomicMass() {
		return weight;
	}

	/**
	 * @return the percentage
	 */
	public double getPercent() {
		return percent;
	}

	/**
	 * sets the atomic mass
	 */
	public void setAtomicMass(double val) {
		weight = val;
		modified = true;
	}

	/**
	 * sets the percentage
	 */
	public boolean setPercent(double val) {
		if (val < 0 || val > 1)
			return false;
		percent = val;
		modified = true;
		return true;
	}

	/**
	 * @return true if the isotope has been modified
	 */
	public boolean isModified() {
		return modified;
	}

	/**
	 * Overriden method; used when dumping the tree
	 */
	public String toString() {
		return "Isotope [" + weight + ":" + (percent * 100) + "%]";
	}
}
