import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.Enumeration;
import java.awt.*;
import java.awt.event.*;

public class DatabaseInterface extends JPanel implements ActionListener {
	// ////////////BEGIN: STATIC CODE///////////////////////
	private static final long serialVersionUID = 3258769155145986147L;

	/**
	 * <p>
	 * Polymorphicaly sorts an array of Objects, the polymorphisim is handled by
	 * the <tt>sort_helper</tt> passed in.
	 * </p>
	 * 
	 * <p>
	 * <tt>ISorter</tt> defines one method, greater than which compares two
	 * elements from the array and determines which is greater.
	 * </p>
	 * 
	 * @param objects
	 *            an array to sort
	 * @param sort_helper
	 *            the <tt>ISorter</tt> that helps sorting the array
	 * @return a sorted array, same array that was passed in
	 */
	private static Object[] Sort(Object[] objects, ISorter sort_helper) {
		boolean sorted;

		do {
			sorted = true;
			for (int i = 0; i < (objects.length - 1); i++) {
				if (sort_helper.greaterthan(objects[i], objects[i + 1])) {
					Object o = objects[i];
					objects[i] = objects[i + 1];
					objects[i + 1] = o;
					sorted = false;
				}
			}
		} while (!sorted);

		return objects;
	}

	// //////////////END: STATIC CODE///////////////////////

	/**
	 * Simple constructor, builds the interface
	 */
	public DatabaseInterface() {
		assemble();
		set_attributes();
		attach_listeners();
	}

	// ////////////BEGIN: UI CONSTRUCTION///////////////////

	private JTable element_table, isotope_table;

	private JButton btnAdd, btnRemove, btnSave, btnRestore, btnAddIsotope,
			btnRemoveIsotope;

	private JPanel pnlCommands, pnlTables, pnlIsotopeCmds, pnlIsotopes;

	private IsotopeTableModel isotope_model;

	private ElementTableModel element_model;

	private void assemble() {
		/* instantiate the UI elements */
		isotope_model = new IsotopeTableModel();
		element_model = new ElementTableModel(isotope_model);
		isotope_table = new JTable(isotope_model);
		element_table = new JTable(element_model);
		btnAdd = new JButton("Add New");
		btnRemove = new JButton("Delete");
		btnSave = new JButton("Save Changes");
		btnRestore = new JButton("Revert To Saved Copy");
		btnAddIsotope = new JButton("Add Isotope");
		btnRemoveIsotope = new JButton("Remove Isotope");
		pnlCommands = new JPanel(new GridLayout(1, 4));
		pnlTables = new JPanel(new FlowLayout());
		pnlIsotopeCmds = new JPanel(new GridLayout(1, 2));
		pnlIsotopes = new JPanel(new BorderLayout());
		/* build the interface */
		setLayout(new BorderLayout());
		add(pnlCommands, BorderLayout.NORTH);
		add(pnlTables, BorderLayout.CENTER);
		pnlCommands.add(btnAdd);
		pnlCommands.add(btnRemove);
		pnlCommands.add(btnSave);
		pnlCommands.add(btnRestore);
		pnlTables.add((new JScrollPane(element_table)));
		pnlTables.add(pnlIsotopes);
		pnlIsotopes.add(new JScrollPane(isotope_table), BorderLayout.CENTER);
		pnlIsotopes.add(pnlIsotopeCmds, BorderLayout.SOUTH);
		pnlIsotopeCmds.add(btnAddIsotope);
		pnlIsotopeCmds.add(btnRemoveIsotope);
	}

	private void set_attributes() {
		/* setup the element table */
		element_table.getTableHeader().setReorderingAllowed(false);
		element_table
				.setPreferredScrollableViewportSize(new Dimension(300, 500));
		element_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		/* setup the isotope table */
		isotope_table.getTableHeader().setReorderingAllowed(false);
		isotope_table
				.setPreferredScrollableViewportSize(new Dimension(150, 400));
	}

	private void attach_listeners() {
		isotope_table.getTableHeader().addMouseListener(isotope_model);
		element_table.getTableHeader().addMouseListener(element_model);
		element_table.getSelectionModel().addListSelectionListener(
				element_model);
		btnAdd.addActionListener(element_model);
		btnRemove.addActionListener(element_model);
		btnAddIsotope.addActionListener(isotope_model);
		btnRemoveIsotope.addActionListener(isotope_model);
		btnSave.addActionListener(this);
		btnRestore.addActionListener(this);
	}

	/**
	 * handles the read and write commands
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == btnSave) {
			IsotopeLibrary.getLibrary().save_library("ISOTOPES");
			IsotopeLibrary.getLibrary().initilise("ISOTOPES");
			element_model.update();
			isotope_model.update(null);
		} else if (e.getSource() == btnRestore) {
			IsotopeLibrary.getLibrary().initilise("ISOTOPES");
			element_model.update();
			isotope_model.update(null);
		}
	}

	// //////////////END: UI CONSTRUCTION///////////////////

	/**
	 * <p>
	 * Encapsulates the functionality of the element table
	 * </p>
	 * <p>
	 * Provides the data for the table, listens for updates and writes the data
	 * back again to the library.
	 * </p>
	 * 
	 * @author Team NAH
	 */
	private class ElementTableModel extends AbstractTableModel implements
			MouseListener, ListSelectionListener, ActionListener {
		private static final long serialVersionUID = 3258769155145986148L;

		private String[] columnNames = { "Element Symbol", "Name",
				"Atomic Number", "Isotope Count" };

		private Element[] data;

		private IsotopeLibrary lib = IsotopeLibrary.getLibrary();

		private int order_by = 2;

		private IsotopeTableModel isotope_model;

		private SimpleElement newElement = new SimpleElement();

		/**
		 * Handles button events that are related to the data of the elements
		 * table.
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == btnAdd)
				addElement();
			else if (e.getSource() == btnRemove)
				removeElement();
		}

		/**
		 * Called by the <tt>TableHeader</tt>'s <tt>MouseListener</tt> when
		 * the column headers are clicked.
		 */
		public void mouseClicked(MouseEvent e) {
			int column = element_table.getTableHeader().columnAtPoint(
					e.getPoint());

			if (column == -1)
				return;

			order_by = column;
			Sort(data, getSorter());
			fireTableDataChanged();
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		/**
		 * adds the new element at the bottom of the element table to the
		 * library.
		 */
		public void addElement() {
			try {
				newElement.validate();
			} catch (IllegalArgumentException ex) {
				ErrorSubject.quickError(UIErrorObserver.UIErrorID, ex
						.getMessage());
				return;
			}
			lib.addElement(newElement);
			newElement = new SimpleElement();
			update();
		}

		/**
		 * removes the selected element from the library
		 */
		public void removeElement() {
			if (element_table.getSelectedRow() == data.length
					|| element_table.getSelectedRow() == -1)
				return;
			lib.removeElement(data[element_table.getSelectedRow()].getSymbol());
			isotope_model.update(null);
			update();
		}

		/**
		 * Simple constructor, reads all the data from the isotope library
		 */
		public ElementTableModel(IsotopeTableModel model) {
			isotope_model = model;
			update();
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return lib.count() + 1;
		}

		public String getColumnName(int col) {
			return columnNames[col];
		}

		/**
		 * looks up a cell in the Element array
		 */
		public Object getValueAt(int row, int col) {
			if (data == null)
				update();
			Element currentElement;
			if (row == data.length)
				currentElement = newElement;
			else
				currentElement = data[row];
			switch (col) {
			case 0: /* Symbol */
				return currentElement.getSymbol();
			case 1: /* Name */
				return currentElement.getName();
			case 2: /* Atomic Number */
				return new Integer(currentElement.getAtomicNumber());
			case 3: /* Isotope Count */
				return new Integer(currentElement.getIsotopes().count());
			}
			return "Unexpected Column Index";
		}

		public Class getColumnClass(int c) {
			switch (c) {
			case 2:
			case 3: /* Atomic Number or Isotope Count */
				return Integer.class;
			}
			/* Any other column is a string */
			return String.class;
		}

		/**
		 * sets if a given cell is editable
		 */
		public boolean isCellEditable(int row, int col) {
			// Note that the data/cell address is constant,
			// no matter where the cell appears onscreen.
			if (row == data.length && col != 3)
				return true;
			return false;
		}

		/**
		 * called every time the user enteres data into the element table
		 */
		public void setValueAt(Object value, int row, int col) {
			try {
				switch (col) {
				case 0: /* Symbol */
					if (!newElement.setSymbol(value.toString()))
						ErrorSubject
								.quickError(UIErrorObserver.UIErrorID,
										"Error setting the symbol, probably already exists");
					break;
				case 1: /* Name */
					newElement.setName(value.toString());
					break;
				case 2: /* Atomic Number */
					if (!newElement.setAtomicNumber(((Integer) value)
							.intValue()))
						ErrorSubject
								.quickError(UIErrorObserver.UIErrorID,
										"Error setting the atomic number, probably already exists");
					break;
				}
			} catch (NullPointerException e) {
			}
			fireTableCellUpdated(row, col);
		}

		/**
		 * Updates the data in the table with new data from the isotope library,
		 * doesnt call <code>fireTableDataChanged</code>.
		 */
		public void update() {
			int i = 0;
			Enumeration enumerator = lib.getEnumeration();

			data = new Element[lib.count()];

			while (enumerator.hasMoreElements()) {
				Object obj = enumerator.nextElement();
				if (obj instanceof Element) {
					data[i] = (Element) obj;
					i++;
				}
			}
			Sort(data, getSorter());
			fireTableStructureChanged();
		}

		/**
		 * @return an instantiated ISorter depending on what value is set in the
		 *         database interface
		 */
		private ISorter getSorter() {
			switch (order_by) {
			case 0: /* Symbol */
				return new SymbolSorter();
			case 1: /* Name */
				return new NameSorter();
			case 2: /* Atomic Number */
				return new AtomicNumberSorter();
			case 3: /* Isotope Count */
				return new IsotopeCountSorter();
			}
			return new SymbolSorter();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
		 */
		public void valueChanged(ListSelectionEvent e) {
			if (!e.getValueIsAdjusting()
					&& element_table.getSelectedRow() != -1) {
				if (element_table.getSelectedRow() == data.length)
					isotope_model.update(newElement);
				else
					isotope_model.update(data[element_table.getSelectedRow()]);
			}

		}
	}

	/**
	 * Models the Element table representation and data.
	 * 
	 * @author Team NAH
	 */
	private class IsotopeTableModel extends AbstractTableModel implements
			MouseListener, ActionListener {
		private static final long serialVersionUID = 3258769155145986149L;

		private String[] columnNames = { "Atomic Mass", "Percentage" };

		private Isotope[] data;

		private Element element;

		private int order_by = 0;

		/**
		 * handles button events that are related to the isotope table
		 * 
		 * @param e
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == btnAddIsotope && element != null) {
				element.getIsotopes().add(new SimpleIsotope());
				update(element);
			} else if (e.getSource() == btnRemoveIsotope && element != null) {
				if (isotope_table.getSelectedRow() != -1 && data.length > 1) {
					element.getIsotopes().remove(
							data[isotope_table.getSelectedRow()]);
					update(element);
				}
			}
		}

		/**
		 * Called by the <tt>TableHeader</tt>'s <tt>MouseListener</tt> when
		 * the column headers are clicked.
		 */
		public void mouseClicked(MouseEvent e) {
			int column = element_table.getTableHeader().columnAtPoint(
					e.getPoint());

			if (column == -1)
				return;

			order_by = column;
			Sort(data, getSorter());
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			if (element != null)
				return element.getIsotopes().count();
			else
				return 0;
		}

		public String getColumnName(int col) {
			return columnNames[col];
		}

		/**
		 * looks up a cell in the Isotope array
		 */
		public Object getValueAt(int row, int col) {
			if (data == null)
				return "Select an element";
			switch (col) {
			case 0: /* Atomic Mass */
				return new Double(data[row].getAtomicMass());
			case 1: /* Percent */
				return new Double(data[row].getPercent());
			}
			return "Unexpected Column Index";
		}

		public Class getColumnClass(int c) {
			/* Only double used in this table */
			if (data != null)
				return Double.class;
			else
				return String.class;
		}

		/**
		 * all elements of the isotope table are editable
		 */
		public boolean isCellEditable(int row, int col) {
			return true;
		}

		/*
		 * Don't need to implement this method unless your table's data can
		 * change.
		 */
		public void setValueAt(Object value, int row, int col) {
			if (col == 0
					&& data[row].getAtomicMass() != ((Double) value)
							.doubleValue())
				data[row].setAtomicMass(((Double) value).doubleValue());
			if (col == 1
					&& data[row].getPercent() != ((Double) value).doubleValue())
				if (!data[row].setPercent(((Double) value).doubleValue()))
					ErrorSubject.quickError(UIErrorObserver.UIErrorID,
							"Tried to set percentage '" + value
									+ "' on isotope " + element.getName() + ":"
									+ data[row].getAtomicMass()
									+ " but was invalid");

			fireTableCellUpdated(row, col);
		}

		/**
		 * Updates the data in the table with new data from isotope list
		 * attached to the element.
		 */
		public void update(Element e) {
			element = e;
			if (e != null) {
				data = element.getIsotopes().toArray();
				Sort(data, getSorter());
			}
			fireTableStructureChanged();
		}

		/**
		 * @return an instantiated ISorter depending on what value is set in the
		 *         database interface
		 */
		private ISorter getSorter() {
			switch (order_by) {
			case 0: /* Atomic Mass */
				return new IsotopeMassSorter();
			case 1: /* Percentage */
				return new IsotopePercentSorter();
			}
			return new IsotopePercentSorter();
		}
	}

	/**
	 * Defines an interface for sorting, aids in making the sorting algorithem
	 * polymorphic.
	 */
	private interface ISorter {
		boolean greaterthan(Object obj1, Object obj2);
	}

	/**
	 * Compares the atomic weight of two elements
	 */
	private class AtomicNumberSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return ((Element) obj1).getAtomicNumber() > ((Element) obj2)
					.getAtomicNumber();
		}
	}

	/**
	 * Compares the symbol of two elements
	 */
	private class SymbolSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return ((Element) obj1).getSymbol().compareTo(
					((Element) obj2).getSymbol()) > 0;
		}
	}

	/**
	 * Compares the name of two elements
	 */
	private class NameSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return ((Element) obj1).getName().compareTo(
					((Element) obj2).getName()) > 0;
		}
	}

	/**
	 * Compares the isotopes counter of two elements
	 */
	private class IsotopeCountSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return ((Element) obj1).getIsotopes().count() > ((Element) obj2)
					.getIsotopes().count();
		}
	}

	/**
	 * Compares the isotope's atomic mass
	 */
	private class IsotopeMassSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return ((Isotope) obj1).getAtomicMass() > ((Isotope) obj2)
					.getAtomicMass();
		}
	}

	/**
	 * Compares the isotopes counter of two elements
	 */
	private class IsotopePercentSorter implements ISorter {
		public boolean greaterthan(Object obj1, Object obj2) {
			return !(((Isotope) obj1).getPercent() > ((Isotope) obj2)
					.getPercent());
		}
	}
}
