import java.util.ArrayList;

public class Formula_Parser
{

    /*************************************
    This class will be given a formula (as a string).
    Then it will turn the string into elements,
    and save them in an an array list.
    The calculator class will then ask for this list.
    **************************************/

    private ArrayList symbol_list;
    private ArrayList element_list;
    private Calculator calculator;
    private ElementContainer root;
    private GraphDataSet graphData;
    
    /**
     * Constructor takes a chemical formula, processes it and sends the result to the calculator.
     * @param formula A string representing the chemical formula (e.g. "H2O")
     */
    public Formula_Parser(String formula) {
        // Process the string.
        symbol_list = expand_formula(formula);
        
        // Convert string symbols to elements.
        element_list = convert_to_elements(symbol_list);
        
        // Create a calculator.
        calculator = new Calculator(element_list);
        
        // Retrieve graph data.
        graphData = calculator.getGraphData();
    }
    
    /**
     * Returns the array list containing the Elements in the formula.
     * @return An ArrayList object containing all the elements derived from the formula.
     */
    public ArrayList get_element_list() {
        return element_list;
    }
    
    /**
     * Expands the formula into single elements and stores them in an ArrayList as strings 
     * eg -  "H2SO4" becomes ["H", "H", "S", "O", "O", "O", "O"]
     */
    public ArrayList expand_formula(String formula) {
    // Note: Some element symbols are 2 letters long so we must be carefull of case e.g.- Mg, Li...
        int index = 0, c = 0;
        //boolean just_had_a_digit = false;
        String temp_element = "" + formula.charAt(index), temp_quantity = "";
        ArrayList array = new ArrayList();
        // I have already added the first letter to the temporary string to avoid
        // mucking up the algorithm later.
        for (index=1; index<formula.length(); index++) {
            // If character is a letter
            if (Character.isLetter(formula.charAt(index))) {
                // if its upper case it means we're starting a new element
                if (Character.isUpperCase(formula.charAt(index))) {
                    // save the old temporary strings and then clear them.
                    if (temp_quantity.equals("")) {temp_quantity = "1";} // If unspecified (eg- "SO") we assume 1 of each ("S", "O").
                    for(c=1; c<=Integer.parseInt(temp_quantity); c++) {
                        array.add(temp_element);
                    }
                    
                    temp_element = "";
                    temp_quantity = "";
                    
                    //just_had_a_digit = false; // Flag that we've just had a letter.
                }
                // Add the new letter to the temporary string.
                temp_element = temp_element + formula.charAt(index);
            }
            // if its a digit
            else {
                temp_quantity = temp_quantity + formula.charAt(index);
                //just_had_a_digit = true; // Flag that we've just had a digit.
            }
        }
        if (temp_quantity.equals("")) {temp_quantity = "1";} // If unspecified (eg- "SO") we assume 1 of each ("S", "O").
        for(c=1; c<=Integer.parseInt(temp_quantity); c++) {
            array.add(temp_element);
        }
        return array;
    }
    
    /**
     * Takes an Array List of strings (e.g.- ["H", "H", "O"] ) and converts it
     * to an Array List of Element objects.
     * 
     * @param array_of_strings An array list of individual element symbols.
     * @return An array list of individual Element objects.
     */
    public ArrayList convert_to_elements(ArrayList array_of_strings) {
        int i=0;
        Element element;
        String symbol;
        ArrayList array_of_elements = new ArrayList();;
        
        for(i=0; i<array_of_strings.size(); i++) {
            symbol = (String)array_of_strings.get(i);
            element = (Element) IsotopeLibrary.getLibrary().getElement(symbol);
            
            array_of_elements.add(element);
        }
        
        return array_of_elements;
    }
    
    /**
     * Returns a set of graph data.
     */
    public GraphDataSet getGraphDataSet() {
        return graphData;
    }

}